# conquest

TAKEOVERTYPE_CAPTURE = 1
TAKEOVERTYPE_NEUTRALIZE = 2

SCORE_CAPTURE = 2
SCORE_NEUTRALIZE = 2
SCORE_CAPTUREASSIST = 1
SCORE_NEUTRALIZEASSIST = 1
SCORE_DEFEND = 1

Top = 0
Middle = 1
Bottom = 2

dmP = {}

streaks = {}
streaks[5] = 1
streaks[6] = 2
streaks[7] = 2
streaks[8] = 2
streaks[9] = 2
streaks[10] = 3

kstreaks = {}
kstreaks[3] = 2
kstreaks[4] = 3
kstreaks[5] = 5

import host
import bf2
import math
from game.scoringCommon import addScore, RPL
from bf2 import g_debug

g_controlPoints = [] # cache, as this map won't change

class pDataObject:
	def __init__(self):
		self.killstreak = 0
		self.knifestreak = 0
	def clearStreaks(self):
		self.killstreak = 0
		self.knifestreak = 0

def init():
	# events hook
	host.registerGameStatusHandler(onGameStatusChanged)
	if host.sgl_getIsAIGame() == 1:
		host.sh_setEnableCommander(0)
	else:
		host.sh_setEnableCommander(0)

	host.registerHandler('TimeLimitReached', onTimeLimitReached, 1)

	if g_debug: print "gpm_cq.py initialized"



def deinit():
	bf2.triggerManager.destroyAllTriggers()
	global g_controlPoints
	g_controlPoints = []
	host.unregisterGameStatusHandler(onGameStatusChanged)
	if g_debug: print "gpm_cq.py uninitialized"



def onGameStatusChanged(status):

	global g_controlPoints
	global dmP
	if status == bf2.GameStatus.Playing:

		# add control point triggers
		g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')
		for obj in g_controlPoints:
			radius = float(obj.getTemplateProperty('radius'))
			isHemi = int(obj.cp_getParam('isHemisphere'))
			if isHemi != 0:
				id = bf2.triggerManager.createHemiSphericalTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))
			else:
				id = bf2.triggerManager.createRadiusTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))
			obj.triggerId = id
			obj.lastAttackingTeam = 0
			if obj.cp_getParam('team') > 0:
				obj.flagPosition = Top
			else:
				obj.flagPosition = Bottom

		# player events
		host.registerHandler('PlayerDeath', onPlayerDeathCQ)
		host.registerHandler('PlayerKilled', onPlayerKilledCQ)
		host.registerHandler('PlayerRevived', onPlayerRevived)
		host.registerHandler('PlayerSpawn', onPlayerSpawn)
		host.registerHandler('EnterVehicle', onEnterVehicle)
		host.registerHandler('ExitVehicle', onExitVehicle)

		bf2.gameLogic.setTickets(1, 999)
		bf2.gameLogic.setTickets(2, 999)

		dmP = {}

		host.rcon_invoke("sv.soldierfriendlyfire 100")
		host.rcon_invoke("sv.vehiclefriendlyfire 100")

		if g_debug: print "DM gamemode initialized."
	else:
		bf2.triggerManager.destroyAllTriggers()
		g_controlPoints = []



def calcStartTickets(mapDefaultTickets):
	return int(mapDefaultTickets * (bf2.serverSettings.getTicketRatio() / 100.0))

def onTimeLimitReached(value):
	host.sgl_endGame(1, 3)

# actual ticket loss calculation function
def calcTicketLossForTeam(team, otherTeamAreaValue, otherTeamAreaOverweight):
	if otherTeamAreaValue >= 100 and otherTeamAreaOverweight > 0:
		ticketLossPerSecond = (bf2.gameLogic.getDefaultTicketLossPerMin(team) / 60.0) * (otherTeamAreaOverweight / 100.0)
		return ticketLossPerSecond
	else:
		return 0



DOWNWARDS = -1
UPWARDS = 1


def updateScore(player,weapon):
	pD = dmP[player.getName()]

	if streaks.has_key(pD.killstreak):
		addScore(player,streaks[pD.killstreak])
		if pD.killstreak == 5:
			sendMsg("5-kill streak!",player)
		elif pD.killstreak == 10:
			sendMsg("10-kill streak!")
	elif pD.killstreak > 10:
		addScore(player,3)

	if weapon == "kni_knife":
		if kstreaks.has_key(pD.knifestreak):
			addScore(player,kstreaks[pD.knifestreak])
			if pD.knifestreak == 3:
				sendMsg("3-kill knife streak!",player)
			elif pD.knifestreak == 5:
				sendMsg("5-kill knife streak!",player)
		elif pD.knifestreak > 5:
			addScore(player,5)
			sendMsg(str(pD.knifestreak) + "-kill knife streak!",player)


def onCPTrigger(triggerId, cp, vehicle, enter, userData):
	pass

# called when tickets reach a predetermined limit (negativ value means that the tickets have become less than the limit)
def onTicketLimitReached(team, limitId):
	pass

# called when a control point flag reached top or bottom
def onCPStatusChange(cp, top):
	pass


def onPlayerDeathCQ(victim, vehicle):
	v = dmP[victim.getName()]
	v.clearStreaks()

def onPlayerKilledCQ(victim, attacker, weapon, assists, object):
	if not victim:
		return
	if attacker == None:
		return

	# killed by remote controlled vehicle, no score awarded in this game
	if object and object.isPlayerControlObject and object.getIsRemoteControlled():
		return

	# no attacker, killed by object
	elif attacker == None:
		return

	# killed by self
	elif attacker == victim:
		return

	else:
		a = dmP[attacker.getName()]
		a.killstreak += 1
		print "Kill streak +1 (" + str(a.killstreak) + ")"
		if weapon:
			print weapon.templateName
			if weapon.templateName == "kni_knife":
				a.knifestreak += 1
				print "Knife streak +1 (" + str(a.knifestreak) + ")"
			elif weapon.templateName == "rurif_dragunov":
				distance = getVectorDistance(attacker.getDefaultVehicle().getPosition(),victim.getDefaultVehicle().getPosition())
				print "Sniper kill - " + str(distance)
				if distance < 10:
					killpoints = -1
				elif distance < 30:
					killpoints = 0
				elif distance >= 30:
					killpoints = math.floor(distance / 30)
				sendMsg(str(int(distance)) + "m Sniper Kill (+" + str(int(killpoints + 1)) + ")",attacker)
				addScore(attacker,killpoints)
		updateScore(attacker,weapon.templateName)

def onPlayerRevived(victim, attacker):
	# update flag takeover status if victim was in a CP radius
	victim.killed = False

def onPlayerSpawn(player, soldier):
	player.killed = False
	if not dmP.has_key(player.getName()):
		dmP[player.getName()] = pDataObject()

def onEnterVehicle(player, vehicle, freeSoldier = False):
	pass

def onExitVehicle(player, vehicle):
	pass

def sendMsg(msg, p=0):
	if p:
		host.rcon_invoke("game.sayAll \"" + str(p.getName()) + " - " + str(msg) + "\"")
	else:
		host.rcon_invoke("game.sayAll \"" + str(msg) + "\"")

# get distance between two positions
def getVectorDistance(pos1, pos2):
	diffVec = [0.0, 0.0, 0.0]
	diffVec[0] = math.fabs(pos1[0] - pos2[0])
	diffVec[1] = math.fabs(pos1[1] - pos2[1])
	diffVec[2] = math.fabs(pos1[2] - pos2[2])

	return math.sqrt(diffVec[0] * diffVec[0] + diffVec[1] * diffVec[1] + diffVec[2] * diffVec[2])



