# common scoring

import host
import bf2
from bf2.stats.constants import *
from bf2 import g_debug

SCORE_KILL = 1
SCORE_TEAMKILL = -4
SCORE_SUICIDE = -2
SCORE_REVIVE = 2
SCORE_TEAMDAMAGE = -2
SCORE_TEAMVEHICLEDAMAGE = -1
SCORE_DESTROYREMOTECONTROLLED = 1

SCORE_KILLASSIST_DRIVER = 1
SCORE_KILLASSIST_PASSENGER = 0
SCORE_KILLASSIST_TARGETER = 1
SCORE_KILLASSIST_DAMAGE = 1

REPAIR_POINT_LIMIT = 100
HEAL_POINT_LIMIT = 100
GIVEAMMO_POINT_LIMIT = 100
TEAMDAMAGE_POINT_LIMIT = 50
TEAMVEHICLEDAMAGE_POINT_LIMIT = 50

REPLENISH_POINT_MIN_INTERVAL = 30	# seconds

# sub score
NORMAL = 0
SKILL = 1
RPL = 2
CMND = 3



def init():

	# set limits for how many repair HPs etc are needed to get a callback
	bf2.gameLogic.setHealPointLimit(HEAL_POINT_LIMIT)
	bf2.gameLogic.setRepairPointLimit(REPAIR_POINT_LIMIT)
	bf2.gameLogic.setGiveAmmoPointLimit(GIVEAMMO_POINT_LIMIT)
	bf2.gameLogic.setTeamDamagePointLimit(TEAMDAMAGE_POINT_LIMIT)
	bf2.gameLogic.setTeamVehicleDamagePointLimit(TEAMVEHICLEDAMAGE_POINT_LIMIT)

	host.registerGameStatusHandler(onGameStatusChanged)

	if g_debug: print "scoring common init"



def onGameStatusChanged(status):
	if status == bf2.GameStatus.Playing:
		host.registerHandler('PlayerKilled', onPlayerKilled)
		host.registerHandler('PlayerDeath', onPlayerDeath)
		host.registerHandler('PlayerRevived', onPlayerRevived)
		host.registerHandler('PlayerHealPoint', onPlayerHealPoint)
		host.registerHandler('PlayerRepairPoint', onPlayerRepairPoint)
		host.registerHandler('PlayerGiveAmmoPoint', onPlayerGiveAmmoPoint)
		host.registerHandler('PlayerTeamDamagePoint', onPlayerTeamDamagePoint)
		host.registerHandler('VehicleDestroyed', onVehicleDestroyed)

	elif status == bf2.GameStatus.EndGame:

		giveCommanderEndScore(bf2.playerManager.getCommander(1), bf2.gameLogic.getWinner())
		giveCommanderEndScore(bf2.playerManager.getCommander(2), bf2.gameLogic.getWinner())



# give commander score for every player score
def addScore(player, points, subScore = NORMAL, subPoints = -1):

	# commander doesnt get score for regular actions, only for pure commander tasks. he also gets punishing points.
	if not player.isCommander() or subScore == CMND or points < 0:
		player.score.score += points
		#if subPoints == -1:
		#	subPoints = points

		# sub score
		#if subScore == RPL:
		#	player.score.rplScore += subPoints
		#if subScore == SKILL:
		#	player.score.skillScore += subPoints
		#if subScore == CMND:
		#	player.score.cmdScore += subPoints

	# commander score
	#commander = bf2.playerManager.getCommander(player.getTeam())
	#if commander != None and commander.isValid() and subScore != CMND and player != commander and points > 0:
	#	preScore = commander.score.score
	#	numPlayers = bf2.playerManager.getNumberOfAlivePlayersInTeam(commander.getTeam())
	#	if numPlayers > 0:
	#		commander.score.score += float(points) / numPlayers
	#		scoreGotten = commander.score.score - preScore
	#		if scoreGotten > 0:
	#			commander.score.cmdScore += scoreGotten



def giveCommanderEndScore(player, winningTeam):
	pass



def onPlayerKilled(victim, attacker, weapon, assists, object):

	killedByEmptyVehicle = False
	countAssists = False

	# killed by unknown, no score
	if attacker == None:

		# check if killed by vehicle in motion
		if weapon == None and object != None:
			if hasattr(object, 'lastDrivingPlayerIndex'):
				attacker = bf2.playerManager.getPlayerByIndex(object.lastDrivingPlayerIndex)
				killedByEmptyVehicle = True


		if attacker == None:
			if g_debug: print "No attacker found"
			pass

	victimVehicle = victim.getVehicle()


	# killed by remote controlled vehicle, no score awarded in this game
	if object and object.isPlayerControlObject and object.getIsRemoteControlled():
		pass

	# no attacker, killed by object
	elif attacker == None:
		pass

	# killed by self
	elif attacker == victim:

		# no suicides from own wreck
		if killedByEmptyVehicle and object.getIsWreck():
			return

		attacker.score.suicides += 1
		attacker.score.skillScore = 0
		addScore(attacker, SCORE_SUICIDE)

	else:
		attacker.score.kills += 1
		attacker.score.skillScore += 1
		addScore(attacker, SCORE_KILL)

		countAssists = True


def onPlayerDeath(victim, vehicle):
	victim.score.deaths += 1
	victim.score.skillScore = 0


def onPlayerRevived(victim, attacker):
	pass



# prevent point-exploiting by replenishing same player again
def checkGrindBlock(player, object):

	if object.isPlayerControlObject:
		defPlayers = object.getOccupyingPlayers()
		if len(defPlayers) > 0:
			defPlayer = defPlayers[0]

			if not hasattr(player, 'lastReplenishPointMap'):
				player.lastReplenishPointMap = {}
			else:
				if defPlayer.index in player.lastReplenishPointMap:
					if player.lastReplenishPointMap[defPlayer.index] + REPLENISH_POINT_MIN_INTERVAL > host.timer_getWallTime():
						return True

			player.lastReplenishPointMap[defPlayer.index] = host.timer_getWallTime()

	return False


def onPlayerHealPoint(player, object):
	pass



def onPlayerRepairPoint(player, object):
	pass



def onPlayerGiveAmmoPoint(player, object):
	pass



def giveDriverSpecialPoint(player):
	pass


def onPlayerTeamDamagePoint(player, object):
	pass



# prevent point-exploiting by replenishing same player again
def checkGrindBlockRemote(player, object):

	if not hasattr(player, 'lastDestroyedRemote'):
		player.lastDestroyedRemote = {}
	else:
		if object in player.lastDestroyedRemote:
			if player.lastDestroyedRemote[object] + REPLENISH_POINT_MIN_INTERVAL > host.timer_getWallTime():
				return True

	player.lastDestroyedRemote[object] = host.timer_getWallTime()
	return False

def onVehicleDestroyed(vehicle, attacker):
	pass


